﻿using CMS;
using CMS.DataEngine;
using CMS.Membership;
using CMS.EventLog;
using Custom;
using Auth0.AuthenticationApi;
using System;
using System.Configuration;
using Auth0.AuthenticationApi.Models;
using System.IdentityModel.Services;
using System.Security.Policy;
using System.Web;

// Registers the custom module into the system
[assembly: RegisterModule(typeof(Custom.CustomAuthenticationModule))]


namespace Custom
{
    public class CustomAuthenticationModule : Module
    {
        // Module class constructor, the system registers the module under the name "CustomAuthentication"
        public CustomAuthenticationModule()
            : base("CustomAuthentication")
        {
        }

        // Contains initialization code that is executed when the application starts
        protected override void OnInit()
        {
            base.OnInit();

            // Assigns a handler to the SecurityEvents.AuthenticationRequested.Execute event
            // This event occurs when users attempt to access a restricted section of Kentico
            SecurityEvents.AuthenticationRequested.Execute += SignIn_Execute;
        }

        // Handler method that writes the URL from which the authentication request was made into the event log
        // You can replace it with your custom code
        private void SignIn_Execute(object sender, AuthenticationRequestEventArgs e)
        {
            var client = new AuthenticationApiClient(
            new Uri(string.Format("https://{0}", ConfigurationManager.AppSettings["auth0:Domain"])));
            
            var authorizeUrlBuilder = client.BuildAuthorizationUrl()
                .WithClient(ConfigurationManager.AppSettings["auth0:ClientId"])
                .WithRedirectUrl(ConfigurationManager.AppSettings["auth0:CallbackURL"])
                .WithResponseType(AuthorizationResponseType.Code)
                .WithScope("openid profile")
                // adding this audience will cause Auth0 to use the OIDC-Conformant pipeline
                // you don't need it if your client is flagged as OIDC-Conformant (Advance Settings | OAuth)
                .WithAudience("https://" + @ConfigurationManager.AppSettings["auth0:Domain"] + "/userinfo");

            if (!string.IsNullOrEmpty(e.RequestedUrl))
            {
                var state = "ru=" + HttpUtility.UrlEncode(e.RequestedUrl);
                authorizeUrlBuilder.WithState(state);
            }
            HttpContext.Current.Response.Redirect(authorizeUrlBuilder.Build().ToString());
        }
    }
}