namespace CMSApp
{
    using System;
    using System.Threading.Tasks;
    using Auth0.AuthenticationApi;
    using Auth0.AuthenticationApi.Models;
    using Auth0.AspNet;
    using System.Collections.Generic;
    using System.Configuration;
    using System.IdentityModel.Services;
    using System.Linq;
    using System.Web;
    using CMS.Membership;

    public class LoginCallback : HttpTaskAsyncHandler
    {
        public override async Task ProcessRequestAsync(HttpContext context)
        {
            AuthenticationApiClient client = new AuthenticationApiClient(
                new Uri(string.Format("https://{0}", ConfigurationManager.AppSettings["auth0:Domain"])));

            var token = await client.GetTokenAsync(new AuthorizationCodeTokenRequest
            {
                ClientId = ConfigurationManager.AppSettings["auth0:ClientId"],
                ClientSecret = ConfigurationManager.AppSettings["auth0:ClientSecret"],
                Code = context.Request.QueryString["code"],
                RedirectUri = context.Request.Url.ToString()
            });

            var profile = await client.GetUserInfoAsync(token.AccessToken);

            // Check if the user exists in Kentico
            CMS.Membership.UserInfo ui = UserInfoProvider.GetUserInfo(profile.NickName + "@gmail.com");
            if(ui == null)
            {
                // Creates a new user object
                CMS.Membership.UserInfo uiNew = new CMS.Membership.UserInfo();

                // Sets the user properties
                uiNew.UserName = profile.NickName + "@gmail.com";
                uiNew.Email = profile.NickName + "@gmail.com";
                uiNew.FirstName = profile.FirstName;
                uiNew.LastName = profile.LastName;
                uiNew.FullName = profile.FullName;
                uiNew.IsExternal = true;
                uiNew.Enabled = true;

                // Saves the user to the database
                UserInfoProvider.SetUserInfo(uiNew);
                ui = uiNew;
            }

            if (ui != null)
            {
                // Log the user in
                AuthenticationHelper.AuthenticateUser(ui.UserName, true);

                var returnTo = "/";
                var state = context.Request.QueryString["state"];
                if (state != null)
                {
                    var stateValues = HttpUtility.ParseQueryString(context.Request.QueryString["state"]);
                    var redirectUrl = stateValues["ru"];                    
                    returnTo = redirectUrl;
                }

                // Redirect to the requested page
                context.Response.Redirect(returnTo);
            }
        }
    }
}